function [] = pickPolygon(src, ~)
%PICKPOLYGON Lets the user pick a polygon in the plot.
%   During selection, a line is being plotted that shows the selected
%   points by the user.
%   The polygon can be stopped by using the right mouse button. If
%   finished, the selected points are being calculated and stored in the
%   artoaWorkspace variable.

global artoaGui artoaWorkspace;

modeName = 'pickPolygon';

%% Check for any active mode
if strcmp(artoaGui.editPressure.currentInteractionMode, modeName)
    % pickPoint mode is active, so finish the mode
    closePickPolygonMode();
    return;
end

if ~islogical(artoaGui.editPressure.currentInteractionMode)
    errordlg('Another interaction mode is currently active. Please finish it first!', 'Interaction mode error!');
    return;
end

%% Check if polygon is already selected
if ~artoa.controller.edit.pressure.dropSelection()
    return;
end

%% Set mode to pickPoint
openPickPolygonMode();

%% Initialize variable
artoaGui.editPressure.userSelection = {};
artoaWorkspace.editPressure.userSelection = false(length(artoaWorkspace.pressure), 1);

%% Initialize polygon
artoaGui.editPressure.tmpPolygon = struct();
artoaGui.editPressure.tmpPolygon.x = [];
artoaGui.editPressure.tmpPolygon.y = [];

% draw the line that will be extended on every click
artoaGui.editPressure.userSelection = line( ...
    NaN, NaN, 'Color', [1 1 1], 'LineStyle', '-' ...
);

button = src;

return

%% Helper functions

    function openPickPolygonMode()
        pan('off');
        zoom('off');
        brush('off');
        
        set(artoaGui.figures.editPressure, 'WindowButtonMotionFcn', @onMouseMove);
        set(artoaGui.figures.editPressure, 'WindowButtonDownFcn', @onMouseAxesDown);
        artoaGui.editPressure.currentInteractionMode = modeName;
        artoaGui.figures.editPressure.Color = [.6 .6 .6];
        button.String = 'Finish';
        drawnow;
    end

    function closePickPolygonMode()
        finishPolygon();
        set(artoaGui.figures.editPressure, 'WindowButtonMotionFcn', '');
        set(artoaGui.figures.editPressure, 'WindowButtonDownFcn', '');
        artoaGui.editPressure.currentInteractionMode = false;
        artoaGui.figures.editPressure.Color = [.94 .94 .94];
        button.String = 'Polygon';
        drawnow;
    end

    function onMouseAxesDown(src, ~)

        if strcmp(src.SelectionType, 'alt')
            closePickPolygonMode();
            return;
        end

        currentPoint = artoaGui.editPressure.axesScatterPressure.CurrentPoint;
        
        % get boundaries
        bound = artoaGui.editPressure.axesScatterPressure.XLim;
        dx = artoa.data.adjustToBoundaries(currentPoint(1, 1), bound(1), bound(2));
        bound = artoaGui.editPressure.axesScatterPressure.YLim;
        dy = artoa.data.adjustToBoundaries(currentPoint(1, 2), bound(1), bound(2));
        
        % add values to array
        artoaGui.editPressure.tmpPolygon.x = [artoaGui.editPressure.tmpPolygon.x; dx];
        artoaGui.editPressure.tmpPolygon.y = [artoaGui.editPressure.tmpPolygon.y; dy];
        
        button.String = ['Finish ' num2str(length(artoaGui.editPressure.tmpPolygon.x)) ' selected'];
        
        if length(artoaGui.editPressure.tmpPolygon.x) == 1
            return;
        end
        % update the line
        set( ...
            artoaGui.editPressure.userSelection, ...
            'XData', artoaGui.editPressure.tmpPolygon.x, ...
            'YData', artoaGui.editPressure.tmpPolygon.y ...
            );
        
    end

    function onMouseMove(~, ~)
        if ~artoa.data.hasMember(artoaGui, 'editPressure', 'tmpPolygon', 'x') ...
                || isempty(artoaGui.editPressure.tmpPolygon.x)
            return;
        end
        
        mousepoint = get(artoaGui.editPressure.axesScatterPressure, 'CurrentPoint');
        set(artoaGui.editPressure.userSelection, ...
            'XData', [artoaGui.editPressure.tmpPolygon.x; mousepoint(1, 1)], ...
            'YData', [artoaGui.editPressure.tmpPolygon.y; mousepoint(1, 2)] ...
            );
        drawnow;
    end

    function finishPolygon()
        % close the poligon by adding the first value as the last one
        if length(artoaGui.editPressure.tmpPolygon.x) > 3 % a polygon can only be closed if there are more than three values
            x = [artoaGui.editPressure.tmpPolygon.x; artoaGui.editPressure.tmpPolygon.x(1)];
            y = [artoaGui.editPressure.tmpPolygon.y; artoaGui.editPressure.tmpPolygon.y(1)];
        else
            [artoaGui.editPressure, artoaWorkspace.editPressure] = artoa.controller.edit.clearSelection(artoaGui.editPressure, artoaWorkspace.editPressure);
            return;
        end
        % update the line
        set(artoaGui.editPressure.userSelection, 'XData', x, 'YData', y);
        % store the selected points in the workspace for editing
        artoaWorkspace.editPressure.userSelection = inpolygon( ...
            artoaWorkspace.rafosDate, ...
            artoaWorkspace.pressure, ...
            x, ...
            y ...
        );
    end

end

