function [ boolean ] = temp_sal_ptemp( p_nemo_data, p_config )
%TEMP_SAL_PTEMP Plots the temperature, salinity and the potential temperature.
%   
%   Parameters:
%   p_nemo_data:        The nemo data to be plotted.
%   p_config:           The nemo configuration.
%
%   Returns:
%   boolean:            True if successful.


%% Initialize return variable
boolean = false;

%% Parameter check %TODO

%% Get config parameter
% get plot details
width = p_config.PLOT_TEMP_SAL_PTEMP.image_width;
height = p_config.PLOT_TEMP_SAL_PTEMP.image_height;
paperunit = p_config.PLOT_TEMP_SAL_PTEMP.image_paperunit;
image_type = p_config.PLOT_TEMP_SAL_PTEMP.image_type;
directory = fullfile(p_config.GENERAL.storage_directory_plots, p_config.PLOT_TEMP_SAL_PTEMP.storage_directory_image);
directory_fig = fullfile(p_config.GENERAL.storage_directory_plots, p_config.PLOT_TEMP_SAL_PTEMP.storage_directory_fig);
ylabel_fontsize = p_config.PLOT_TEMP_SAL_PTEMP.ylabel_fontsize;
xlabel_fontsize = p_config.PLOT_TEMP_SAL_PTEMP.xlabel_fontsize;
axis_fontsize = p_config.PLOT_TEMP_SAL_PTEMP.axis_fontsize;

if (~isdir(directory))
    mkdir(directory);
end
if ~exist(directory_fig, 'dir')
    mkdir(directory_fig);
end

%% Collect required data

switch (class(p_nemo_data{1}))
    case 'struct' % only one float is to be processed
        boolean = plot_float(p_nemo_data);
    case 'cell' % more than one float has to be processed
        boolean = all(cellfun(@plot_float, p_nemo_data));
    otherwise
        disp([mfilename ': Class ' class(p_nemo_data{1}) ' not supported for plotting!']);
end

    function boolean = plot_float(p_float)
        boolean = false;
        profile_data = nemo.float.collect(p_float, 'PROFILE_DATA', '');
        combined_profile_data = nemo.float.collect(p_float, 'PROFILE_DATA', '', 'combine');
        best_guess_lat = nemo.float.collect(p_float, 'SECOND_ORDER_INFORMATION', 'best_guess_position_lat');
        best_guess_lon = nemo.float.collect(p_float, 'SECOND_ORDER_INFORMATION', 'best_guess_position_lon');
        
        max_pressure = max(combined_profile_data(:, 7));
        max_salinity = max(combined_profile_data(:, 9));
        max_temperature = max(combined_profile_data(:, 5));
        min_pressure = min(combined_profile_data(:, 7));
        min_salinity = min(combined_profile_data(:, 9));
        min_temperature = min(combined_profile_data(:, 5));
        
        if (isempty(profile_data))
            boolean = true;
            return
        end
        
        % initialize figure
        figure_id = figure('Visible', 'Off');
        set(figure_id,'InvertHardcopy','on');
        set(figure_id,'PaperUnits', paperunit, 'Position', [ 0 0 width height]);
        papersize = get(figure_id, 'PaperSize');
        left = (papersize(1) - width)/2;
        bottom = (papersize(2) - height)/2;
        myfiguresize = [left, bottom, width, height];
        set(figure_id,'PaperPosition', myfiguresize);
        
        % Salinity plot
        sub(1) = subplot(1, 3, 1);
        hold on
        set(gca, 'YLimMode', 'manual', ...
                 'YLim', [min_pressure max_pressure], ...
                 'YTick', [min_pressure:200:max_pressure], ...
                 'YDir', 'reverse', ...
                 'XLimMode', 'manual', ...
                 'XLim', [min_salinity max_salinity], ...
                 'XTickMode', 'auto');
        for i = 1:length(profile_data)
            pressure = profile_data{i}(:, 7);
            salinity = profile_data{i}(:, 9);
            plot(salinity, pressure, ...
                 'color', [0 0 1] * i/length(profile_data));
        end
        box on
        grid on
        hold off
        
        % create main title
        t = title(...
             [ 'Float ' ...
               num2str(p_float{1}.FLOAT_IDENTIFICATION.float_serial_number) ...
               ' @ ' ...
               datestr(datetime('now', 'Format', 'dd-MMM-yyyy HH:mm:SS')) ...
             ]);
        set(t, 'FontSize', 18);
        clear t
        xlabel('Salinity [PSU]', 'FontSize', xlabel_fontsize);
        ylabel('Depth [m]', 'FontSize', ylabel_fontsize);
        set(gca, 'fontsize', axis_fontsize);
%         set(gca, 'YDir', 'reverse', 'YTick', [0:2000]);
%         ylim([-2000 0]);
        
%         set(gca, 'YLimMode', 'manual', 'YLim', [-2000 0], 'YTick', [-2000:200:0]);
        
        clear salinity pressure a
        
        % Temperature plot
        sub(2) = subplot(1, 3, 2);
        hold on
        set(gca, 'YLimMode', 'manual', ...
                 'YLim', [min_pressure max_pressure], ...
                 'YTick', [min_pressure:200:max_pressure], ...
                 'YDir', 'reverse', ...
                 'XLimMode', 'manual', ...
                 'XLim', [min_temperature, max_temperature], ...
                 'XTickMode', 'auto');
        for i = 1:length(profile_data)
            pressure = profile_data{i}(:, 7);
            temperature = profile_data{i}(:, 5);
            plot(temperature, pressure, ...
                 'color', [1 0 0] * i/length(profile_data));
        end
        xlabel('Temperature [C]', 'FontSize', xlabel_fontsize);
        ylabel('Depth [m]', 'FontSize', ylabel_fontsize);
        set(gca, 'fontsize', axis_fontsize);
        box on
        grid on
        hold off
        clear pressure temperature a
        
        % Potential temperature plot
        sub(3) = subplot(2, 3, 3);
        hold on
        set(gca, 'XLimMode', 'manual', ...
                 'XLim', [min_salinity max_salinity], ...
                 'XTick', [min_salinity:max_salinity], ...
                 'XTickMode', 'auto', ...
                 'YLimMode', 'manual', ...
                 'YTickMode', 'auto', ...
                 'YLim', [min_temperature, max_temperature]);
        for i = 1:length(profile_data)
            pressure = profile_data{i}(:, 7);
            temperature = profile_data{i}(:, 5);
            salinity = profile_data{i}(:, 9);
            plot(salinity, sw_ptmp(salinity, temperature, pressure, 0), ...
                         'Linestyle', 'none', ...
                         'color', [0 1 0] * i/length(profile_data), ...
                         'marker','.' ...
                );
        end
        ylabel('\theta (Potential temperature) [C]', 'FontSize', xlabel_fontsize);
        xlabel('Salinity [PSU]', 'FontSize', ylabel_fontsize);
        set(gca, 'fontsize', axis_fontsize);
        box on
        grid on
        hold off
        clear a
        
        sub(4) = subplot(2, 3, 6);
        hold on
        latlim = [-80 80];
        lonlim = [-90 70];
        worldmap(latlim,lonlim);
        setm(gca, 'MLabelParallel', 'south')
        if (~exist('map', 'var'))
            map = load('coast');
        end
        plotm(map.lat,map.long,'k');
        plot3m(best_guess_lat,best_guess_lon,150,'marker','.','markersize',8,'color',[0 1 0],'linestyle','none');
        xlabel('Longitude', 'FontSize', xlabel_fontsize);
        ylabel('Latitude', 'FontSize', ylabel_fontsize);
        set(gca, 'fontsize', axis_fontsize);
        box on
        grid on
        hold off
        clear a
        
         
        set(gcf, 'Renderer', 'painters');
        print(figure_id, ...
         [directory p_config.PLOT_TEMP_SAL_PTEMP.image_filename ...
           num2str(p_float{1}.FLOAT_IDENTIFICATION.float_serial_number)] ...
         , image_type);

        if (p_config.PLOT_TEMP_SAL_PTEMP.save_figure)
            set(gcf,'CreateFcn','set(gcf,''Visible'',''on'')');
            saveas(figure_id, ...
                [directory_fig ...
                 p_config.PLOT_TEMP_SAL_PTEMP.figure_filename ...
                 num2str(p_float{1}.FLOAT_IDENTIFICATION.float_serial_number)]);
        end

        close(figure_id);
        clear figure_id f
        boolean = true;
        
    end




end

